#include "PrimitiveGenerator.h"
#include <cmath>

using core::types::Vec3;

namespace core::utils {

    void Primitive::CreateQuad(float width, float height,
                               std::vector<Vec3>& outVerts, std::vector<uint32_t>& outIndices)
    {
        outVerts.clear(); outIndices.clear();
        float hw = width * 0.5f, hh = height * 0.5f;

        outVerts.emplace_back(-hw, -hh, 0.0f);
        outVerts.emplace_back( hw, -hh, 0.0f);
        outVerts.emplace_back( hw,  hh, 0.0f);
        outVerts.emplace_back(-hw,  hh, 0.0f);

        outIndices = { 0, 1, 2,
              2, 3, 0 };
    }

    void Primitive::CreateCircle(float radius, int segments,
                                 std::vector<Vec3>& outVerts, std::vector<uint32_t>& outIndices)
    {
        outVerts.clear(); outIndices.clear();
        if (segments < 3) segments = 3;

        outVerts.emplace_back(0.0f, 0.0f, 0.0f);

        const float step = 2.0f * static_cast<float>(M_PI) / static_cast<float>(segments);
        for (int i = 0; i <= segments; ++i) {
            float a = step * static_cast<float>(i);
            outVerts.emplace_back(std::cos(a) * radius,
                           std::sin(a) * radius,
                           0.0f);
        }

        for (uint32_t i = 1; i < static_cast<uint32_t>(segments + 1); ++i) {
            outIndices.push_back(0);
            outIndices.push_back(i);
            outIndices.push_back(i+1);
        }
    }

} // namespace core::utils
